#### Function ####
pop.handling<- function(data,axis,list.names, na.omit= F, na.start= 1, graph= F, out= F) {
  axis= unlist(strsplit(axis, ":")) # separa a string para obtenção dos valores representativos do eixo (se linha ou coluna ─ l ou c ─, e qual a posição ordinal da coluna/linha), removendo a configuração automática da variável como lista pelo R;
  margin= ifelse(axis[1]== "l",1, ifelse(axis[1]=="c",2, NA)) #atribuindo 1 ao caractere "l" para considerar que os nomes estão em linha no arquivo, ou 2 para coluna "c", resultando em mensagem de erro caso o usuário não forneça o input solicitado;
  if(margin != 1 & margin != 2) {
    stop("Identificação da linha ou coluna incorreta. Inserir \"l\" para linha ou \"c\" para coluna.\nExemplo: axis = \"c:2\" indica que os nomes estão em uma coluna (segunda coluna).") #mensagem de erro caso o usuário não forneça as letras "l" ou "c" para identificação como linha ou coluna. Obs: observar acima que eu converti as letras para 1 ou 2 para "l" ou "c", respectivamente, e por isso está nesta condição if como 1 ou 2;
  }
  i= suppressWarnings(as.integer(axis[2])) ##atribuindo a posição da linha/coluna indicada pelo usuário ao objeto "i", alterando a classe de caractere para classe dos números inteiros e, se não for (o que deve ser), suprime o aviso de coerção que daria, visto que irei imprimir mensagem erro manualmente com mais detalhes a seguir;
  if(is.na(i)) {
    stop("Identificação da posição da linha/coluna incorreta. Inserir um número inteiro.\nExemplo: em axis = \"c:2\", o número inteiro \"2\" indica a segunda coluna.") #mensagem de erro caso o usuário não forneça números inteiros para identificação da posição da linha/coluna;
  }
  
  list.names= unlist(list.names, use.names = F)  #transformando a lista de nomes em um vetor para utilização/comparação a posteriori;
  
  ## Caso o usuário determine que os nomes estão na linha x (pos) do arquivo matriz/data.frame ##
  if(margin== 1){
    index=which(data[i,] %in% list.names) #comparação dos nomes da lista fornecido pelo usuário aos nomes do arquivo, na linha indicada pelo usuário no argumento "axis", e guardando os indíces dos matchs no objeto "index";
    new.data= data[,index] #criação do objeto contendo todos os dados apenas dos indivíduos/SNPs de interesse do usuário (objetivo principal da função);
    
    #Omitindo NAs (se o usuário desejar)
    if(isTRUE(na.omit)){
      new.data[na.start:nrow(new.data),]= na.omit(new.data[na.start:nrow(new.data),]) #se o parâmetro é TRUE para o argumento na.omit, a função na.omit() será aplicada diretamente nas colunas (onde estão os dados no caso de margin== 1)
    }
    if(!is.na(na.omit)){
      new.data[na.start:nrow(new.data),]=as.data.frame(sapply(new.data[na.start:nrow(new.data),], gsub, pattern = na.omit, replacement = NA)) #substitui o parâmetro que o usuário quiser definir como NA para NAs propriamente ditos;
      new.data=na.omit(new.data) #então omite-se normalmente  os NAs com a função na.omit()
    }
    
    #Criando os gráficos de proporçõa/comparação
    if(graph == T){
    x11(width = 13, height = 6) #abrindo nova janela de plot retangular
    
      # Se NA data.frame de entrada possui (length) maior que zero, gera-se dois gráficos:
      if(is.na(na.omit) & length(which(is.na(data[na.start:ncol(data),]))) > 0){
        par(mfrow= c(1,2), cex= 0.5) #separando janelas em uma linha e duas colunas (uma para cada gráfico)
        pie.graph_IDs= c(length(list.names),length(data[i,]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a porcentagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção  de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao primeiro gráfico;
        pie.graph_NAs= c(length(which(is.na(data[na.start:ncol(data),]))),length(which(is.na(new.data[na.start:ncol(new.data),])))) # contando o tanto de NAs nos data.frames de entrada e saída;
        pie(pie.graph_NAs,labels = paste0(pie.graph_NAs, " NAs"), col= c(gray(0.4),gray(0.8)), main= "Proporção de NAs nos dataframes", cex= 2, cex.main= 2.5, density = 40) #criação do gráfico de pizza para mostrar a quantidade de NAs em ambos os data.frames;
        legend("bottomleft", c("NAs do data.frame de saída","NAs do data.frame de entrada"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao segundo gráfico;
      }
      # Se maior NA é numérico (e.g. = 0) e sua soma é maior que zero, gera-se dois gráficos:
      if(is.numeric(na.omit) & sum(c(sum(grep(na.omit, data[na.start:nrow(data),])), sum(grep(na.omit, new.data[na.start:nrow(new.data),])))) > 0){
        par(mfrow= c(1,2), cex= 0.5) #separando janelas em uma linha e duas colunas (uma para cada gráfico)
        pie.graph_IDs= c(length(list.names),length(data[i,]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a porcentagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais do arquivo"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao primeiro gráfico;
        pie.graph_NAs= c(sum(length(grep(na.omit,data[na.start:nrow(data),]))),sum(length(grep(na.omit,new.data[na.start:nrow(new.data),])))) # contando o tanto de NAs nos data.frames de entrada e saída;
        pie(pie.graph_NAs,labels = paste0(pie.graph_NAs, " NAs"), col= c(gray(0.4),gray(0.8)), main= "Proporção de NAs nos dataframes", cex= 2, cex.main= 2.5, density = 40) #criação do gráfico de pizza para mostrar a quantidade de NAs em ambos os data.frames;
        legend("bottomleft", c("NAs do data.frame de saída","NAs do data.frame de entrada"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao segundo gráfico;
      }  
      # Se NA não foi específicado, gera-se apenas um gráfico, visto que o segundo exigiria um número positivo de NAs
      if(!is.na(na.omit) & !is.numeric(na.omit)) {
        par(mfrow= c(1,1), cex= 0.5) #determina-se apenas uma janela para o gráfico a ser gerado;
        pie.graph_IDs= c(length(list.names),length(data[i,]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a porcentagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais do arquivo"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao gráfico ;
      }  
    }
    
    #Salvando o data.frame
    if(out != F){
      write.table(new.data,out, sep= " ", quote = F, row.names = F, col.names = F) #salvando novo data.frame com path e nome definido pelo usuário;
    }
  }
  
  ## Caso o usuário determine que os nomes estão na coluna x (pos) do arquivo matriz/data.frame ##
  if(margin== 2){
    index=which(data[,i] %in% list.names) #comparação dos nomes da lista fornecido pelo usuário aos nomes do arquivo, na coluna indicada pelo usuário no argumento "axis", e guardando os indíces dos matchs no objeto "index";
    new.data= data[index,] #criação do objeto contendo todos os dados apenas dos indivíduos/SNPs de interesse do usuário (objetivo principal da função);
    
    #Omitindo NAs (se o usuário desejar)
    if(isTRUE(na.omit)){
      new.data[,na.start:ncol(new.data)]= na.omit(new.data[,na.start:ncol(new.data)]) #se o parâmetro é TRUE para o argumento na.omit, a função na.omit() será aplicada diretamente nas linhas (onde estão os dados no caso de margin== 2)
    }
    if(!is.na(na.omit)){
      new.data[,na.start:ncol(new.data)]=as.data.frame(sapply(new.data[,na.start:ncol(new.data)], gsub, pattern = na.omit, replacement = NA)) #substitui o parâmetro que o usuário quiser definir como NA para NAs propriamente ditos;
      new.data=na.omit(new.data) #então omite-se normalmente  os NAs com a função na.omit()
    }
    
    #Criando os gráficos de proporçõa/comparação
    if(graph == T){
    x11(width = 13, height = 6) #abrindo nova janela de plot retangular
      
      # Se NA data.frame de entrada possui (length) maior que zero, gera-se dois gráficos:
      if(is.na(na.omit) & length(which(is.na(data[,na.start:ncol(data)]))) > 0){
        par(mfrow= c(1,2), cex= 0.5) #separando janelas em uma linha e das colunas (uma para cada gráfico)
        pie.graph_IDs= c(length(list.names),length(data[,i]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a portencagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais do arquivo"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao primeiro gráfico
        pie.graph_NAs= c(length(which(is.na(data[,na.start:ncol(data)]))),length(which(is.na(new.data[,na.start:ncol(new.data)])))) # contando o tanto de NAs nos data.frames de entrada e saída
        pie(pie.graph_NAs,labels = paste0(pie.graph_NAs, " NAs"), col= c(gray(0.4),gray(0.8)), main= "Proporção de NAs nos dataframes", cex= 2, cex.main= 2.5, density = 40) #criação do gráfico de pizza para mostrar a quantidade de NAs em ambos os data.frames;
        legend("bottomleft", c("NAs do data.frame de saída","NAs do data.frame de entrada"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao segundo gráfico
      }
      # Se maior NA é numérico (e.g. = 0) e sua soma é maior que zero, gera-se dois gráficos:
      if(is.numeric(na.omit) & sum(c(sum(grep(na.omit, data[,na.start:ncol(data)])), sum(grep(na.omit, new.data[,na.start:ncol(new.data)])))) > 0){
        par(mfrow= c(1,2), cex= 0.5) #separando janelas em uma linha e das colunas (uma para cada gráfico)
        pie.graph_IDs= c(length(list.names),length(data[,i]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a portencagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais do arquivo"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao primeiro gráfico
        pie.graph_NAs= c(sum(length(grep(na.omit,data[,na.start:ncol(data)]))),sum(length(grep(na.omit,new.data[,na.start:ncol(new.data)])))) # contando o tanto de NAs nos data.frames de entrada e saída
        pie(pie.graph_NAs,labels = paste0(pie.graph_NAs, " NAs"), col= c(gray(0.4),gray(0.8)), main= "Proporção de NAs nos dataframes", cex= 2, cex.main= 2.5, density = 40) #criação do gráfico de pizza para mostrar a quantidade de NAs em ambos os data.frames;
        legend("bottomleft", c("NAs do data.frame de saída","NAs do data.frame de entrada"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao segundo gráfico
      }  

      # Se NA não foi específicado, gera-se apenas um gráfico, visto que o segundo exigiria um número positivo de NAs
      if(!is.na(na.omit) & !is.numeric(na.omit)) {
        par(mfrow= c(1,1), cex= 0.5) #determina-se apenas uma janela para o gráfico a ser gerado;
        pie.graph_IDs= c(length(list.names),length(data[,i]))#contabilizando os nomes do data.frame original e do data.frame gerado pela função e colocando-os concatenados em um vetor;
        pie.percent= paste0(round(100*pie.graph_IDs/sum(pie.graph_IDs), 1),"%")#obtendo a porcentagem dos nomes do novo data.frame em relação ao original (entrada do usuário), e adicionando o símbolo de porcentagem à frente para utilização na legenda a seguir;
        pie(pie.graph_IDs,labels = pie.percent, col= c(gray(0.8),gray(0.4)), main= expression(bold(paste("Nomes fornecidos em lista ", italic("versus "),"nomes totais"))), cex= 2, cex.main= 2.5, density = 40) #criação de um gráfico de pizza para mostrar a proporção de nomes na lista fornecida em relação aos totais do data.frame de entrada;
        legend("bottomleft", c("Nomes da lista","Nomes do arquivo original"), cex= 1.75, fill= c(gray(0.8),gray(0.4)))#adicionando legenda ao primeiro gráfico  
      }  
  }
    
    #Salvando o data.frame
    if(out != F){
      write.table(new.data,out, sep= " ", quote = F, row.names = F, col.names = F) #salvando novo data.frame com path e nome definido pelo usuário
    }
  }
  
  return(new.data)
}

### THE END ###